/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

/* NautilusUndoStackManager - Manages undo of file operations (header)
 *
 * Copyright (C) 2007-2010 Amos Brocco
 *
 * Author: Amos Brocco <amos.brocco@unifr.ch>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef NAUTILUS_UNDOSTACK_MANAGER_H
#define NAUTILUS_UNDOSTACK_MANAGER_H

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtkwidget.h>
#include <gio/gio.h>

/* Begin action structures */

typedef enum
{
  NAUTILUS_UNDOSTACK_COPY,
  NAUTILUS_UNDOSTACK_DUPLICATE,
  NAUTILUS_UNDOSTACK_MOVE,
  NAUTILUS_UNDOSTACK_RENAME,
  NAUTILUS_UNDOSTACK_CREATEEMPTYFILE,
  NAUTILUS_UNDOSTACK_CREATEFILEFROMTEMPLATE,
  NAUTILUS_UNDOSTACK_CREATEFOLDER,
  NAUTILUS_UNDOSTACK_MOVETOTRASH,
  NAUTILUS_UNDOSTACK_CREATELINK,
  NAUTILUS_UNDOSTACK_DELETE,
  NAUTILUS_UNDOSTACK_RESTOREFROMTRASH,
  NAUTILUS_UNDOSTACK_SETPERMISSIONS,
  NAUTILUS_UNDOSTACK_RECURSIVESETPERMISSIONS,
  NAUTILUS_UNDOSTACK_CHANGEOWNER,
  NAUTILUS_UNDOSTACK_CHANGEGROUP
} NautilusUndoStackActionType;

typedef struct _NautilusUndoStackActionData NautilusUndoStackActionData;

typedef struct _NautilusUndoStackMenuData NautilusUndoStackMenuData;

struct _NautilusUndoStackMenuData {
  char* undo_label;
  char* undo_description;
  char* redo_label;
  char* redo_description;
};

/* End action structures */

typedef void
(*NautilusUndostackFinishCallback)(gpointer data);

typedef struct _NautilusUndoStackManagerPrivate NautilusUndoStackManagerPrivate;

typedef struct _NautilusUndoStackManager
{
  GObject parent_instance;

  NautilusUndoStackManagerPrivate* priv;

} NautilusUndoStackManager;

typedef struct _NautilusUndoStackManagerClass
{
  GObjectClass parent_class;
  
} NautilusUndoStackManagerClass;

#define TYPE_NAUTILUS_UNDOSTACK_MANAGER (nautilus_undostack_manager_get_type())

#define NAUTILUS_UNDOSTACK_MANAGER(object) \
 (G_TYPE_CHECK_INSTANCE_CAST((object), TYPE_NAUTILUS_UNDOSTACK_MANAGER, NautilusUndoStackManager))

#define NAUTILUS_UNDOSTACK_MANAGER_CLASS(klass) \
 (G_TYPE_CHECK_CLASS_CAST((klass), TYPE_NAUTILUS_UNDOSTACK_MANAGER, NautilusUndoStackManagerClass))

#define IS_NAUTILUS_UNDOSTACK_MANAGER(object) \
 (G_TYPE_CHECK_INSTANCE_TYPE((object), TYPE_NAUTILUS_UNDOSTACK_MANAGER))

#define IS_NAUTILUS_UNDOSTACK_MANAGER_CLASS(klass) \
 (G_TYPE_CHECK_CLASS_TYPE((klass), TYPE_NAUTILUS_UNDOSTACK_MANAGER))

#define NAUTILUS_UNDOSTACK_MANAGER_GET_CLASS(object) \
 (G_TYPE_INSTANCE_GET_CLASS((object), TYPE_NAUTILUS_UNDOSTACK_MANAGER, NautilusUndoStackManagerClass))
 
GType
nautilus_undostack_manager_get_type (void);

void
nautilus_undostack_manager_add_action(NautilusUndoStackManager* manager,
    NautilusUndoStackActionData* action);

void
nautilus_undostack_manager_undo(NautilusUndoStackManager* manager,
    GtkWidget *parent_view, NautilusUndostackFinishCallback cb);

void
nautilus_undostack_manager_redo(NautilusUndoStackManager* manager,
    GtkWidget *parent_view, NautilusUndostackFinishCallback cb);

NautilusUndoStackActionData*
nautilus_undostack_manager_data_new(NautilusUndoStackActionType type,
    gint items_count);

gboolean
nautilus_undostack_manager_is_undo_redo(NautilusUndoStackManager* manager);

void
nautilus_undostack_manager_trash_has_emptied(NautilusUndoStackManager* manager);

NautilusUndoStackManager*
nautilus_undostack_manager_instance(void);

void
nautilus_undostack_manager_data_set_src_dir(NautilusUndoStackActionData* data,
    GFile* src);

void
nautilus_undostack_manager_data_set_dest_dir(NautilusUndoStackActionData* data,
    GFile* dest);

void
nautilus_undostack_manager_data_add_origin_target_pair(
    NautilusUndoStackActionData* data, GFile* origin, GFile* target);

void
nautilus_undostack_manager_data_set_create_data(
    NautilusUndoStackActionData* data, char* target_uri, char* template_uri);

void
nautilus_undostack_manager_data_set_rename_information(
    NautilusUndoStackActionData* data, GFile* old_file, GFile* new_file);

guint64
nautilus_undostack_manager_get_file_modification_time(GFile* file);

void
nautilus_undostack_manager_data_add_trashed_file(
    NautilusUndoStackActionData* data, GFile* file, guint64 mtime);

void
nautilus_undostack_manager_request_menu_update(NautilusUndoStackManager* manager);

void
nautilus_undostack_manager_data_add_file_permissions(
    NautilusUndoStackActionData* data, GFile* file, guint32 permission);
    
void
nautilus_undostack_manager_data_set_recursive_permissions(
    NautilusUndoStackActionData* data, guint32 file_permissions, guint32 file_mask,
	guint32 dir_permissions, guint32 dir_mask);
    
void
nautilus_undostack_manager_data_set_file_permissions(
    NautilusUndoStackActionData* data, char* uri, guint32 current_permissions, guint32 new_permissions);
    
void
nautilus_undostack_manager_data_set_owner_change_information(
    NautilusUndoStackActionData* data, char* uri, const char* current_user, const char* new_user);
    
void
nautilus_undostack_manager_data_set_group_change_information(
    NautilusUndoStackActionData* data, char* uri, const char* current_group, const char* new_group);    

#endif /* NAUTILUS_UNDOSTACK_MANAGER_H */
